@@ -11,7 +11,8 @@
 #define ATTS_AC_NOT_CONFIGURED  4
 #define ATTS_NOT_LOGGED_IN      5
 #define ATTS_LOGIN_FAIL_SHOWURL 6
-#define ATTS_TIMEDOUT           7
+#define ATTS_NW_ERROR           7
+#define ATTS_USER_CANCEL        8
 
 class AsyncTaskToken : public QObject
 {
