@@ -220,6 +220,7 @@ GVApi::doGet(QUrl url, AsyncTaskToken *token, QObject *receiver, const char *met
 
     tracker->setAutoRedirect (jar, UA_IPHONE4, true);
     token->apiCtx = tracker;
+    token->status = ATTS_SUCCESS;
 
     bool rv =
     connect(tracker, SIGNAL (sigDone(bool, const QByteArray &, QNetworkReply *,
@@ -270,6 +271,7 @@ GVApi::doPost(QUrl url, QByteArray postData, const char *contentType,
 
     tracker->setAutoRedirect (jar, ua, true);
     token->apiCtx = tracker;
+    token->status = ATTS_SUCCESS;
 
     bool rv = connect(tracker, SIGNAL(sigDone(bool, const QByteArray &,
                                               QNetworkReply *, void *)),
@@ -370,10 +372,10 @@ void
 GVApi::cancel(AsyncTaskToken *token)
 {
     NwReqTracker *tracker = (NwReqTracker *)token->apiCtx;
-    if (tracker) {
+    if (tracker == NULL) {
         Q_WARN("API context not valid. Cannot cancel. I can at least fail it");
 
-        token->status = ATTS_FAILURE;
+        token->status = ATTS_USER_CANCEL;
         token->emitCompleted ();
         return;
     }
@@ -431,7 +433,10 @@ GVApi::onLogin1(bool success, const QByteArray &response, QNetworkReply *,
     QString strResponse = response;
 
     do { // Begin cleanup block (not a loop)
-        if (!success) break;
+        if (!success) {
+            token->status = ATTS_NW_ERROR;
+            break;
+        }
 
         // We may have completed login already. Check for cookie "gvx"
         foreach (QNetworkCookie cookie, jar->getAllCookies ()) {
@@ -461,10 +466,7 @@ GVApi::onLogin1(bool success, const QByteArray &response, QNetworkReply *,
     if (!success) {
         Q_WARN(QString("Login failed: %1").arg(strResponse));
 
-        NwReqTracker *tracker = (NwReqTracker *) this->sender ();
-        if (tracker->isTimedOut ()) {
-            token->status = ATTS_TIMEDOUT;
-        } else {
+        if (token->status == ATTS_SUCCESS) {
             token->status = ATTS_LOGIN_FAILURE;
         }
         token->emitCompleted ();
@@ -598,11 +600,13 @@ GVApi::onLogin2(bool success, const QByteArray &response, QNetworkReply *reply,
     bool accountConfigured = true;
     bool accountReviewRequested = false;
     bool foreignUrl = false;
-    NwReqTracker *tracker = (NwReqTracker *) this->sender ();
 
     token->errorString.clear();
     do { // Begin cleanup block (not a loop)
-        if (!success) break;
+        if (!success) {
+            token->status = ATTS_NW_ERROR;
+            break;
+        }
 
         // There will be 2-3 moved temporarily redirects.
         QUrl urlMoved = NwReqTracker::hasMoved(reply);
@@ -758,25 +762,20 @@ GVApi::onLogin2(bool success, const QByteArray &response, QNetworkReply *reply,
     } while (0); // End cleanup block (not a loop)
 
     if (!success) {
-        if (accountConfigured) {
+        if (token->status == ATTS_NW_ERROR) {
+        } else if (accountConfigured) {
             Q_WARN("Login failed.") << strResponse;
 
             if (token->errorString.isEmpty()) {
                 token->errorString = tr("The username or password you entered "
                                         "is incorrect.");
             }
-            if (tracker->isTimedOut ()) {
-                token->status = ATTS_TIMEDOUT;
-            } else {
-                token->status = ATTS_LOGIN_FAILURE;
-            }
-            token->emitCompleted ();
+            token->status = ATTS_LOGIN_FAILURE;
         }
         else if (accountReviewRequested) {
             token->errorString = "User login failed: Account recovery "
                                  "requested by Google";
             token->status = ATTS_LOGIN_FAIL_SHOWURL;
-            token->emitCompleted ();
         } else {
             Q_WARN("Login failed because user account was not configured.");
 
@@ -786,8 +785,10 @@ GVApi::onLogin2(bool success, const QByteArray &response, QNetworkReply *reply,
                                     "browser and complete the setup of your "
                                     "Google Voice account.");
             token->status = ATTS_AC_NOT_CONFIGURED;
-            token->emitCompleted ();
         }
+
+        // In all cases, emit completed
+        token->emitCompleted ();
     }
 }//GVApi::onLogin2
 
@@ -857,7 +858,10 @@ GVApi::onTFAAutoPost(bool success, const QByteArray &response,
     QString strResponse = response;
 
     do { // Begin cleanup block (not a loop)
-        if (!success) break;
+        if (!success) {
+            token->status = ATTS_NW_ERROR;
+            break;
+        }
 
         success = false;
         QRegExp rx("<form\\s*action\\s*=\\s*\"(.*)\"\\s*method\\s*=\\s*\"POST\"");
@@ -886,7 +890,9 @@ GVApi::onTFAAutoPost(bool success, const QByteArray &response,
     if (!success) {
         Q_WARN("Login failed.") << strResponse;
 
-        token->status = ATTS_LOGIN_FAILURE;
+        if (token->status == ATTS_SUCCESS) {
+            token->status = ATTS_LOGIN_FAILURE;
+        }
         token->emitCompleted ();
     }
 }//GVApi::onTFAAutoPost
@@ -912,7 +918,10 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *reply,
     QString strReplyUrl = reply->url().toString();
 
     do { // Begin cleanup block (not a loop)
-        if (!success) break;
+        if (!success) {
+            token->status = ATTS_NW_ERROR;
+            break;
+        }
 
         success = false;
         int pos = strResponse.indexOf ("_rnr_se");
@@ -980,10 +989,7 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *reply,
         Q_WARN(msg);
 
         if (token) {
-            NwReqTracker *tracker = (NwReqTracker *) this->sender ();
-            if (tracker->isTimedOut ()) {
-                token->status = ATTS_TIMEDOUT;
-            } else {
+            if (token->status == ATTS_SUCCESS) {
                 token->status = ATTS_LOGIN_FAILURE;
             }
             token->emitCompleted ();
@@ -1069,7 +1075,7 @@ GVApi::onGetPhones(bool success, const QByteArray &response, QNetworkReply *,
 
     do { // Begin cleanup block (not a loop)
         if (!success) {
-            Q_WARN("Failed to get phones");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -1232,7 +1238,9 @@ GVApi::onGetPhones(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -1278,15 +1286,19 @@ GVApi::onGetInbox(bool success, const QByteArray &response, QNetworkReply *,
                   void *ctx)
 {
     AsyncTaskToken *token = (AsyncTaskToken *)ctx;
-    QString strReply = response;
+    QString strReply = QString::fromUtf8(response.constData(),response.length());
 
     do { // Begin cleanup block (not a loop)
         if (!success) {
-            Q_WARN("Failed to get inbox");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
 
+#if 0
+        Q_DEBUG(strReply);
+#endif
+
         QXmlInputSource inputSource;
         QXmlSimpleReader simpleReader;
         inputSource.setData (strReply);
@@ -1302,8 +1314,7 @@ GVApi::onGetInbox(bool success, const QByteArray &response, QNetworkReply *,
         }
 
         qint32 msgCount = 0;
-        if (!parseInboxJson (xmlHandler.strJson, xmlHandler.strHtml, msgCount))
-        {
+        if (!parseInboxJson(xmlHandler.strJson, xmlHandler.strHtml, msgCount)) {
             Q_WARN("Failed to parse GV Inbox JSON. Data =") << strReply;
             break;
         }
@@ -1318,7 +1329,9 @@ GVApi::onGetInbox(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -1429,6 +1442,8 @@ GVApi::parseInboxJson(const QString &strJson, const QString &strHtml,
                         inboxEntry.Type = GVIE_Voicemail;
                     } else if (strVal.contains ("sms")) {
                         inboxEntry.Type = GVIE_TextMessage;
+                    } else if (strVal.contains ("trash")) {
+                        inboxEntry.bTrash = true;
                     } else {
                         if (emitLog) {
                             Q_WARN("Unknown label") << strVal;
@@ -1474,8 +1489,8 @@ GVApi::parseInboxJson(const QString &strJson, const QString &strHtml,
             }
 
             // Pick up the text from the parsed HTML
-            if (((GVIE_TextMessage == inboxEntry.Type) ||
-                 (GVIE_Voicemail == inboxEntry.Type)))
+            if ((GVIE_TextMessage == inboxEntry.Type) ||
+                (GVIE_Voicemail   == inboxEntry.Type))
             {
                 QString strQuery =
                     QString("for $i in doc('%1')//div[@id=\"%2\"]\n"
@@ -1497,7 +1512,7 @@ GVApi::parseInboxJson(const QString &strJson, const QString &strHtml,
                         "  return $i")
                         .arg (fSms.fileName ());
                 if (!execXQuery (strQuery, resultSms)) {
-                    Q_WARN("XQuery failed for Text :") << strQuery;
+                    Q_WARN(QString("XQuery failed for Text: %1").arg(strQuery));
                 }
                 resultSms = resultSms.trimmed ();
 
@@ -1540,13 +1555,35 @@ GVApi::execXQuery(const QString &strQuery, QString &result)
     if (!xQuery.isValid() || !xQuery.evaluateTo (&formatter)) {
         return false;
     }
-    result = outArray;
+    result = QString::fromUtf8(outArray.constData(),outArray.length());
 
     return true;
 }//GVApi::execXQuery
 
+static void
+fixAmpersandEncoded(QString &strTemp)
+{
+    strTemp.replace ("&amp", "&");
+    QRegExp rx("&#(.*)\\;");
+    rx.setMinimal (true);
+    while (strTemp.contains (rx)) {
+        bool bOk;
+        QString strHex = rx.cap(0).remove("#").remove(";")
+                .remove("&");
+        char iVal = strHex.toInt (&bOk);
+        strTemp.replace (rx.cap (0), QString(iVal));
+    }
+}//fixAmpersandEncoded
+
+/** Parse the HTML document for a message row
+ * @param strRow The text of the html
+ * @param entry The inbox entry generated out of this function (OUT).
+ *
+ * strRow should have
+ *  <div class="gc-message-message-display" ...> ... </div>
+ */
 bool
-GVApi::parseMessageRow(QString &strRow, GVInboxEntry &entry)
+GVApi::parseMessageRow(QString strRow, GVInboxEntry &entry)
 {
     bool rv = false;
     QString strSmsRow;
@@ -1555,68 +1592,135 @@ GVApi::parseMessageRow(QString &strRow, GVInboxEntry &entry)
     do { // Begin cleanup block (not a loop)
         QDomDocument doc;
         doc.setContent (strRow);
-        QDomElement topElement = doc.childNodes ().at (0).toElement ();
-        if (topElement.isNull ()) {
-            Q_WARN ("Top element is null");
+        QDomElement rootElement = doc.documentElement();
+        if (rootElement.isNull ()) {
+            Q_WARN("Top element is null");
             break;
         }
 
-        // Children could be either SMS rows or vmail transcription
         QDomNamedNodeMap attrs;
-        strSmsRow.clear ();
+        QDomNodeList divNodes, spanNodes;
+        QDomNode oneSpan;
+        QDomAttr oneAttr;
+        QString attrValue, strTemp;
 
-        QDomNodeList smsRow = topElement.childNodes();
-        for (int j = 0; j < smsRow.size (); j++) {
-            if (!smsRow.at(j).isElement()) {
+        // Pick up all "span"s to look for voicemail translations
+        spanNodes = rootElement.elementsByTagName ("span");
+        if (spanNodes.isEmpty ()) {
+            Q_WARN("No span nodes!");
+            break;
+        }
+
+        // Loop through all spans looking for the attribute "class"
+        rv = false;
+        for (int i = 0; i < spanNodes.count(); i++) {
+            oneSpan = spanNodes.at(i);
+
+            attrs = oneSpan.attributes ();
+            if (!attrs.contains ("class")) {
                 continue;
             }
 
-            QDomElement smsSpan = smsRow.at(j).toElement();
-            if (smsSpan.tagName () != "span") {
+            oneAttr = attrs.namedItem("class").toAttr();
+            if (!oneAttr.isAttr ()) {
+                Q_WARN("Invalid attribute");
+                continue;
+            }
+
+            attrValue = oneAttr.value();
+            if (attrValue == "gc-edited-trans-text") {
+                //Q_DEBUG("Found a voicemail translation!");
+                convEntry.init ();
+                convEntry.text = oneSpan.toElement().text().simplified();
+                entry.conversation.append (convEntry);
+                rv = true;
+                break;
+            }
+        }
+
+        if (rv) {
+            break;
+        }
+
+        // Now look for "div"s with class="gc-message-sms-row"
+        divNodes = rootElement.elementsByTagName ("div");
+        strSmsRow.clear ();
+
+        // Loop through all divs looking for the attribute "class"
+        rv = false;
+        for (int i = 0; i < divNodes.count(); i++) {
+            QDomNode oneDiv = divNodes.at(i);
+
+            attrs = oneDiv.attributes ();
+            if (!attrs.contains ("class")) {
+                continue;
+            }
+
+            oneAttr = attrs.namedItem("class").toAttr();
+            if (!oneAttr.isAttr ()) {
+                Q_WARN("Invalid attribute");
+                continue;
+            }
+
+            attrValue = oneAttr.value();
+            if (attrValue != "gc-message-sms-row") {
+                Q_DEBUG(QString("Uninteresting div class: %1").arg(attrValue));
+                continue;
+            }
+
+            spanNodes = oneDiv.toElement().elementsByTagName ("span");
+            if (spanNodes.isEmpty ()) {
+                //Q_WARN("No span nodes!");
                 continue;
             }
 
             convEntry.init();
 
-            attrs = smsSpan.attributes();
-            for (int m = 0; m < attrs.size (); m++) {
-                QString strTemp = smsSpan.text ().simplified ();
-                QDomAttr attr = attrs.item(m).toAttr();
-                if (attr.value() == "gc-message-sms-from") {
+            // Loop through all spans looking for the attribute "class"
+            rv = false;
+            for (int i = 0; i < spanNodes.count(); i++) {
+                oneSpan = spanNodes.at(i);
+
+                attrs = oneSpan.attributes ();
+                if (!attrs.contains ("class")) {
+                    continue;
+                }
+
+                oneAttr = attrs.namedItem("class").toAttr();
+                if (!oneAttr.isAttr ()) {
+                    Q_WARN("Invalid attribute");
+                    continue;
+                }
+
+                strTemp = oneSpan.toElement().text().simplified ();
+                attrValue = oneAttr.value();
+                if (attrValue == "gc-message-sms-from") {
                     strSmsRow += "<b>" + strTemp + "</b> ";
                     convEntry.from = strTemp;
-                } else if (attr.value() == "gc-message-sms-text") {
+                } else if (attrValue == "gc-message-sms-text") {
                     strSmsRow += strTemp;
                     convEntry.text += strTemp;
-                } else if (attr.value() == "gc-message-sms-time") {
+                } else if (attrValue == "gc-message-sms-time") {
                     strSmsRow += " <i>(" + strTemp + ")</i><br>";
                     convEntry.time = strTemp;
-                } else if (attr.value().startsWith ("gc-word-")) {
+                } else if (attrValue.startsWith ("gc-word-")) {
                     if (!strSmsRow.isEmpty ()) {
                         strSmsRow += ' ';
                         convEntry.text += ' ';
                     }
 
-                    strTemp.replace ("&amp", "&");
-                    QRegExp rx("&#(.*)\\;");
-                    rx.setMinimal (true);
-                    while (strTemp.contains (rx)) {
-                        bool bOk;
-                        QString strHex = rx.cap(0).remove("#").remove(";")
-                                                  .remove("&");
-                        char iVal = strHex.toInt (&bOk);
-                        strTemp.replace (rx.cap (0), QString(iVal));
-                    }
                     strSmsRow += strTemp;
                     convEntry.text += strTemp;
                 }
-            }// loop thru the parts of a single sms
+            }// for loop thru all spans in the div
 
+            fixAmpersandEncoded (convEntry.text);
             entry.conversation.append (convEntry);
-        }//loop through sms row
+        }//for loop thru all divs with class="gc-message-sms-row"
+
+        fixAmpersandEncoded (strSmsRow);
 
         entry.strText = strSmsRow;
-
         rv = true;
     } while (0); // End cleanup block (not a loop)
 
@@ -1693,6 +1797,7 @@ GVApi::onCallout(bool success, const QByteArray &response, QNetworkReply *,
     do { // Begin cleanup block (not a loop)
         if (!success) {
             Q_WARN("Failed to call out");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -1727,7 +1832,9 @@ GVApi::onCallout(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -1799,6 +1906,7 @@ GVApi::onCallback(bool success, const QByteArray &response, QNetworkReply *,
     do { // Begin cleanup block (not a loop)
         if (!success) {
             Q_WARN("Failed to call back");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -1834,7 +1942,9 @@ GVApi::onCallback(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -1908,6 +2018,7 @@ GVApi::onSendSms(bool success, const QByteArray &response, QNetworkReply *,
     do { // Begin cleanup block (not a loop)
         if (!success) {
             Q_WARN("Failed to send text");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -1944,7 +2055,9 @@ GVApi::onSendSms(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -1988,7 +2101,8 @@ GVApi::onVmail(bool success, const QByteArray &response, QNetworkReply *,
 
     do { // Begin cleanup block (not a loop)
         if (!success) {
-            Q_WARN("Failed to send text");
+            Q_WARN("Failed to get voicemail");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -2017,7 +2131,9 @@ GVApi::onVmail(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -2068,6 +2184,7 @@ GVApi::onMarkAsRead(bool success, const QByteArray &response, QNetworkReply *,
     do { // Begin cleanup block (not a loop)
         if (!success) {
             Q_WARN("Failed to mark entry as read");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -2103,7 +2220,9 @@ GVApi::onMarkAsRead(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -2160,6 +2279,7 @@ GVApi::onEntryDeleted(bool success, const QByteArray &response, QNetworkReply *,
     do { // Begin cleanup block (not a loop)
         if (!success) {
             Q_WARN("Failed to delete entry");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -2195,7 +2315,9 @@ GVApi::onEntryDeleted(bool success, const QByteArray &response, QNetworkReply *,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
@@ -2233,6 +2355,7 @@ GVApi::onCheckRecentInbox(bool success, const QByteArray &response,
     do { // Begin cleanup block (not a loop)
         if (!success) {
             Q_WARN("Failed to get recent inbox");
+            token->status = ATTS_NW_ERROR;
             break;
         }
         success = false;
@@ -2294,7 +2417,7 @@ GVApi::onCheckRecentInbox(bool success, const QByteArray &response,
 
             serverLatest = QDateTime::fromTime_t (iVal);
         } else {
-            Q_DEBUG("Empty list");
+            //Q_DEBUG("Empty list");
             serverLatest = QDateTime::fromMSecsSinceEpoch (0);
         }
 
@@ -2329,7 +2452,9 @@ GVApi::onCheckRecentInbox(bool success, const QByteArray &response,
 
     if (!success) {
         if (token) {
-            token->status = ATTS_FAILURE;
+            if (token->status == ATTS_SUCCESS) {
+                token->status = ATTS_FAILURE;
+            }
             token->emitCompleted ();
         }
     }
