@@ -45,7 +45,8 @@ MainWindow::MainWindow (QWidget *parent)
 , actDismiss ("Dismiss", this)
 , actRefresh ("Refresh", this)
 , actExit ("Exit", this)
-, bLoggedIn (false)
+, loginStatus (LS_NotLoggedIn)
+, loginTask(NULL)
 , modelRegNumber (NULL)
 , indRegPhone (0)
 , mtxDial (QMutex::Recursive)
@@ -700,6 +701,10 @@ MainWindow::connectSettingsSignals()
                       this, SLOT   (doLogout ()));
         Q_ASSERT(rv);
         if (!rv) { break; }
+        rv = connect (obj , SIGNAL (sigCancelLogin ()),
+                      this, SLOT   (doCancelLogin ()));
+        Q_ASSERT(rv);
+        if (!rv) { break; }
 
         obj = getQMLObject ("RefreshSettingsPage");
         if (obj == NULL) {
@@ -822,7 +827,7 @@ MainWindow::refreshRegisteredNumbers ()
 
     bool rv = false;
     do { // Begin cleanup block (not a loop)
-        if (!bLoggedIn) {
+        if (loginStatus != LS_LoggedIn) {
             Q_WARN("Not logged in. Will not refresh registered numbers.");
             break;
         }
